% MATLAB script comparing the reference and MapReduce implementations
clear;

A = load('output_sing.txt');

fid = fopen('output_ref.dat','r');
B = fread(fid,inf,'float');
fclose(fid);

% part-00000 must be copied from S3 or HDFS prior to running this script
fid = fopen('part-00000','r');
tmp = fread(fid,inf,'char');
fclose(fid);
% remove extra formatting for easy loading in MATLAB
tmp2 = [ tmp(4:end-2); tmp(end) ];
%save file and open with LOAD
fid = fopen('temp','w');
fwrite(fid,tmp2,'char');
C = load('temp');
fclose(fid);
!rm temp

A = reshape(A,256,256);
B = reshape(B,256,256);
C = reshape(C,256,256);

idx=find(B>0);
% need to round off B to 9 decimal points for consistency with A
B = round(B*1e9)*1e-9;

cla;clf;hold off;
subplot(2,2,1);
imagesc( log10(1+A) );
axis image; axis off;
title('MapReduce - Console');

subplot(2,2,2);
imagesc( log10(1+C), [0 3.5] );
axis image; axis off;
title('MapReduce - Remote');

subplot(2,2,3);
imagesc( log10(1+B), [0 3.5] );
axis image; axis off;
title('Reference');

subplot(2,2,4);
imagesc( log10(abs(C-B)) );
axis image; axis off; colorbar;
title('Difference');


% Mean-square error
% Should be = 9.9807e-17
MSE = sqrt(mean( ( (A(idx)-B(idx)) ./ B(idx) ).^2 ) );
disp(['Ref / Local: Mean-square error = ' num2str(MSE)]);

MSE = sqrt(mean( ( (C(idx)-B(idx)) ./ B(idx) ).^2 ) );
disp(['Ref / Remote: Mean-square error = ' num2str(MSE)]);

