
/********************************************
 *  mcconv.c by Guillem Pratx
 *
 *  Converts a TypedBytes input produced from a Hadoop-compatible
 *  Reduce task, into human-readable ASCII stream.
 *  This small utility is intended for debugging purposes only,
 *  when running the code locally in a console.
 *
 *  An article describes the use of this program in a cloud computing
 *  environment:
 *    G. Pratx & L. Xing, "Monte Carlo Simulation of Photon Migration 
 *    in a Cloud Computing Environment with MapReduce", 
 *    J. Biomed. Opt. in press. 
 * 
 *  The MC321 package, which is used in this work, was developped by
 *  Steven L. Jacques based on prior collaborative work with Lihong
 *  Wang, Scott Prahl, and Marleen Keijzer.
 *
 *  A published report illustrates use of the MC321 program:
 *    S. L. Jacques: "Light distributions from point, line, and plane 
 *    sources for photochemical reactions and fluorescence in turbid 
 *    biological tissues," Photochem. Photobiol. 67:23-32, 1998. 
 **********/

#include <stdio.h>
#include <stdlib.h>

/* Converts a little-endian unsigned integer into the big-endian
format (and vice-versa) */ 
unsigned int ByteSwap( unsigned int n)
{
return	( (( n << 24) & 0xFF000000) |	\
	  (( n <<  8) & 0x00FF0000) |	\
	  (( n >>  8) & 0x0000FF00) |	\
	  (( n >> 24) & 0x000000FF) );
}

/* Casts a float as an unsigned integer */
unsigned int float_as_int (float f)
{
   return (*((int*)&f));
}

/* Casts an unsigned integer as a float */
float int_as_float (unsigned int i)
{
   return (*((float*)&i));
}

/* Variable convention: The _ suffix indicates a big-endian variable */
float         *value;
unsigned int  *value_,
              key,
              key_,
              keyold,
              keyold_,
              l,
              l_;
char          c;
int           r=1,
              i;

int main(int argc, const char* argv[] ) {

c = fgetc(stdin);
r = fread(&key_, sizeof(int), 1, stdin);
key=ByteSwap(key_);

c = fgetc(stdin);
r = fread(&l_, sizeof(float), 1, stdin);
l = ByteSwap(l_);

value = (float*) calloc( l, sizeof(float) );
value_ = (unsigned int*) calloc( l, sizeof(float) );

for (i=0; i<l; i++) {
  c = fgetc(stdin);
  r = fread(&value_[i], sizeof(unsigned int), 1, stdin);
}

for (i=0; i<l; i++) {
  value[i] = int_as_float(ByteSwap(value_[i]));
  fprintf(stdout, "%.9f ", value[i]);
}

fprintf(stdout, "\n");

free(value);
free(value_);

return 0;

}
